;;; -*- TI-Asm -*-

;; PluginInit:
;;
;; Search for installed plugins and build a table of pointers to them.
;; Note that plugins must be archived - PluginCall will intentionally
;; avoid calling unarchived plugins, because if a plugin is not
;; archived, the pointer will not be updated when the plugin is moved
;; in RAM.  After calling Arc_Unarc, or any other system routine that
;; might move variables in Flash, PluginInit must be called again.
;;
;; Output:
;; - pluginTable contains pointers to installed plugins
;;
;; Destroys:
;; - AF, BC, DE, HL
;; - OP1, ramCode

PluginInit:
	ld hl,pluginTable
	ld b,128
	BCALL _ClrLp
	ld de,(progPtr)
PluginInit_Loop:
	ld hl,(pTemp)
	or a
	sbc hl,de
	ret nc
	ld a,(de)
	cp AppVarObj
	jr nz,PluginInit_Next
	push de
	 ex de,hl
	 call GetVarDataStart
	 ld de,OP1
	 ld bc,9
	 call FlashToRAM
	 push af
	  push hl
	   ld hl,OP1 + 2
	   ld de,pluginFileSignature
	   ld b,6
	   call CompStrsN
	   pop de
	  pop bc
	 jr nz,PluginInit_Next1
	 ld l,(hl)
	 ld h,high(pluginTable)
	 ld (hl),b
	 inc hl
	 ld (hl),e
	 inc hl
	 ld (hl),d
PluginInit_Next1:
	 pop de
PluginInit_Next:
	ex de, hl
	call MoveToNextProgSym
	ex de, hl
	jr PluginInit_Loop

;; PluginCall:
;;
;; Call a plugin if it is installed.
;;
;; Input:
;; - C = plugin table index (3 * plugin ID)
;; - pluginTable initialized by PluginInit
;;
;; Output:
;; - Carry flag set if plugin is not installed
;;
;; Destroys:
;; - AF, B

PluginCall:
	ld b,high(pluginTable)
	ld a,(bc)
	cp 1
	ret c
	push hl
	 push de
	  ld d,a
	  inc c
	  ld a,(bc)
	  ld l,a
	  inc c
	  ld a,(bc)
	  ld h,a
	  ld a,d
	  call Load_DE_AHL
	  ld b,d
	  ld c,e
	  ld de,pluginExecMem
	  call FlashToRAM
	  pop de
	 pop hl
	jp pluginExecMem + 1
