;;; -*- TI-Asm -*-
;;;
;;; Mimas plugin to display library symbols menu
;;;
;;; Copyright (C) 2012 Benjamin Moody
;;;
;;; This program is free software: you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License as
;;; published by the Free Software Foundation; either version 3 of the
;;; License, or (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

	processor z80
	nolist
	include <ti83plus.inc>
	list

MIMAS_PLUGIN equ 1

	nolist
	include <mimas.inc>
	include <defs.inc>
	include <insts.inc>
	include <plugapi.inc>
	list

;; PLUGIN_LIBS:
;;
;; This plugin is called in two cases:
;;
;; - When kWindow is pressed in the assembly editor, the plugin is
;;   called with HL = address of the menu structure to be displayed.
;;   The plugin returns an altered menu structure with the "Libraries"
;;   item appended.
;;
;; - When kCatalog is pressed, or the "Libraries" menu item is
;;   selected, the plugin is called with H = 0.  It displays a menu of
;;   libraries (if any are loaded); if one is selected, it then
;;   returns to EditSectionAsm_PopupAlphaMenu to display a menu of
;;   symbols from that library.
;;
;; Input:
;; - (case 1) HL = asmEditBuiltinMenuInfo
;; - (case 2) H = 0
;;
;; Destroys:
;; - AF, BC, DE, HL, IX
;; - OP1-OP6
;; - saveSScreen

    BEGIN_PLUGIN PLUGIN_LIBS

	ld a,h
	or a
	jr z,LibrarySymbolMenu
	ld de,menuInfoBuffer
	push de
	 ld bc,asmEditBuiltinMenuSize
	 ldir
	 ld hl,importEditLabel
	 ld c,importEditLabelSize
	 ldir
	 pop hl
	inc (hl)
	ret

menuInfoBuffer:

LibrarySymbolMenu:
	call ShowLibraryMenu
	ret c

	ld bc,9
	call GetProgramData
	rst rMOV9TOOP1
	BCALL _OP1ToOP6
	ld hl,LibraryError
	call APP_PUSH_ERRORH
	 call LoadTempProgram
	 call APP_POP_ERRORH

	;; If file contains a constant table (precompiled header), use
	;; that instead of string table
	ld hl,(curProgHeader + PROGHEADER_CONSTANT_START)
	inc hl
	inc hl
	inc hl			; skip sentinel
	ld (alphaMenuTabStart),hl
	ex de,hl
	ld hl,(curProgHeader + PROGHEADER_CONSTANT_END)
	ld (alphaMenuTabEnd),hl
	scf
	sbc hl,de
	ld bc,Extract_oI16SStr
	ld de,SeekCompare_oI16SStr_pZStr
	jr nc,PopupAlphaMenu

	;; Otherwise, use the string table, much as the builtin Lbl
	;; menu does.  However, in this case, we want to omit any
	;; local symbols from the menu.

	;; Fortunately, all locals begin with a dot, so they appear
	;; before all globals beginning with a letter; unfortunately,
	;; globals can also begin with an underscore, which (for no
	;; good reason) is sorted earlier than a dot.  Thus we need to
	;; identify the start and end of the "local string" section,
	;; and use a special seek/compare function that skips over
	;; that section if necessary.

	;; Furthermore, (alphaMenuTabStart) must be set to the offset
	;; of the first global in the file (which might be either
	;; (SYMBOL_STRING_START)+1 or (localSymStrEnd) depending on
	;; whether there are any globals starting with underscores.)

	ld bc,(curProgHeader + PROGHEADER_SYMBOL_STRING_END)
	ld (alphaMenuTabEnd),bc
	ld hl,(curProgHeader + PROGHEADER_SYMBOL_STRING_START)
	inc hl			; skip sentinel
	push hl
	 ld ix,SeekCompare_oI8SStr_pZStr
	 ld de,spacebyte
	 push bc
	  call BSearchS
	  ld (localSymStrStart),hl
	  pop bc
	 ld de,slashbyte
	 call BSearchS
	 ld (localSymStrEnd),hl
	 pop hl
	call SkipLocalSyms
	ld (alphaMenuTabStart),hl

	ld bc,Extract_oI8SStr
	ld de,SeekCompare_oI8SStrGlobal_pZStr
PopupAlphaMenu:
	pop af			;GETRETURN
	jp EditSectionAsm_PopupAlphaMenu

LibraryError:
	;; kludge: if library can't be loaded, (customErrorParam) will
	;; point to curProgFileName + 1, but this is the wrong name
	ld hl,OP6+1
	ld (customErrorParam),hl
	call ShowErrorPopup
	pop af			;GETRETURN
	jp EditSectionAsm_AbortAlphaMenu

;; SeekCompare_oI8SStrGlobal_pZStr:
;;
;; Seek/compare function for user symbol strings (0-nibble-terminated
;; packed strings preceded by an 8-bit symbol-number hint.)  This will
;; also skip local symbols (symbols beginning with a dot.)
;;
;; Input:
;; - HL = program offset within a symbol string
;; - DE = address of zero-terminated string
;; - (localSymStrStart) = offset to start of local symbol area
;; - (localSymStrEnd) = offset to end of local symbol area
;;
;; Output:
;; - Zero flag set, and HL = start of string, if HL "equals" DE
;; - Zero and carry flags reset, and HL = start of string, if HL is
;;   "greater" than DE
;; - Zero flag reset, carry flag set, and HL = end of string, if HL is
;;   "less" than DE
;;
;; Destroys:
;; - AF, BC

SeekCompare_oI8SStrGlobal_pZStr:
localSymStrEnd equ $ + 1
	ld bc,0
	or a
	sbc hl,bc
	add hl,bc
	jp nc,SeekCompare_oI8SStr_pZStr
localSymStrStart equ $ + 1
	ld bc,0
	dec bc
	or a
	sbc hl,bc
	add hl,bc
	jr c,SeekCompare_oI8SStrGlobal_pZStr_NotLocal
	ld h,b
	ld l,c
SeekCompare_oI8SStrGlobal_pZStr_NotLocal:
	call SeekCompare_oI8SStr_pZStr
	ret nc
SkipLocalSyms:
	ld bc,(localSymStrStart)
	xor a
	sbc hl,bc
	add hl,bc
	ret c
	ld hl,(localSymStrEnd)
	inc a
	scf
	ret

;; SeekCompare_oI16SStr_pZStr:
;;
;; Seek/compare function for precompiled constants
;; (0-nibble-terminated packed strings preceded by a 16-bit value.)
;;
;; Input:
;; - HL = program offset within a constant string
;; - DE = address of zero-terminated string
;; - (localSymStrStart) = offset to start of local symbol area
;; - (localSymStrEnd) = offset to end of local symbol area
;;
;; Output:
;; - Zero flag set, and HL = start of string, if HL "equals" DE
;; - Zero and carry flags reset, and HL = start of string, if HL is
;;   "greater" than DE
;; - Zero flag reset, carry flag set, and HL = end of string, if HL is
;;   "less" than DE
;;
;; Destroys:
;; - AF, BC

SeekCompare_oI16SStr_pZStr:
	;; seek backwards to find previous byte with low nibble clear
	dec hl
	call GetProgramDataByte
	and 0Fh
	jr nz,SeekCompare_oI16SStr_pZStr
	;; check if either of the two bytes preceding has low nibble
	;; clear (in case we found a value byte rather than the actual
	;; end of a symbol)
	dec hl
	dec hl
SeekCompare_oI16SStr_pZStr_SeekLoop:
	call GetProgramDataByte
	inc hl
	and 0Fh
	jr nz,SeekCompare_oI16SStr_pZStr_SeekLoop
	call SeekCompare_oI8SStr_pZStr_Main
	ret c
	dec hl
	ret

Extract_oI16SStr:
	inc hl
	jp Extract_oI8SStr

spacebyte: db ' '		; '_' < ' ' < '.' (in symbol sorting order)
slashbyte: db '/'		; '.' < '/' < 'A'

;; ShowLibraryMenu:
;;
;; Display a popup menu allowing the user to select from the list of
;; libraries that are imported by the current program.  The first ten
;; items in the list are numbered and can be selected by pressing the
;; corresponding number key.
;;
;; Output:
;; - Carry flag set if menu cancelled
;; - HL = offset to selected entry in the program's import table
;;
;; Destroys:
;; - AF, BC, DE, HL, IX
;; - OP1
;; - minPenCol
;; - Contents of LCD and saveSScreen

ShowLibraryMenu:
	ld hl,(curProgHeader + PROGHEADER_IMPORT_END)
	ld de,(curProgHeader + PROGHEADER_IMPORT_START)
	xor a
	sbc hl,de
	ret c
	scf
	ret z
	call PopupInit
	ld a,9
	BCALL _DivHLByA
	ld (libraryMenuCount),hl
	ld hl,0
	ld d,h
	ld e,l
ShowLibraryMenu_Loop:
        VERTICAL_MENU_ALPHA 11, 6, 23, 69, LibraryMenuInit, LibraryMenuShow, LibraryMenuNext, LibraryMenuPrev, LibraryMenuInvCursor, LibraryMenuInvCursor
	ld bc,9
	SWITCH_BEGIN
	SWITCH_CASE kEnter,  ShowLibraryMenu_Accept
	SWITCH_CASE k0,      ShowLibraryMenu_NumberKey
	SWITCH_CASE kClear,  ShowLibraryMenu_Cancel
	SWITCH_CASE kQuit,   ShowLibraryMenu_Cancel
	SWITCH_END
	sub k1
        jr c,ShowLibraryMenu_Loop
	cp c
        jr nc,ShowLibraryMenu_Loop
	ld c,a
ShowLibraryMenu_NumberKey:
	push hl
	 ld hl,(libraryMenuCount)
	 scf
	 sbc hl,bc
	 pop hl
	jr c,ShowLibraryMenu_Loop
	ld h,b
	ld l,c
ShowLibraryMenu_Accept:
	call PopupExit
LibraryMenuGetOffset:
	BCALL _HLTimes9
	ld de,(curProgHeader + PROGHEADER_IMPORT_START)
	add hl,de
	ret

ShowLibraryMenu_Cancel:
	scf
	jp PopupExit

LibraryMenuInit:
	ld bc,3406h
	jp DrawPopupBox

LibraryMenuShow:
	ld bc,9
	or a
	sbc hl,bc
	add hl,bc
	ld a,'0'
	jr z,LibraryMenuShow_LabelChar
	jr nc,LibraryMenuShow_NoLabelChar
	adc a,l
	jr LibraryMenuShow_LabelChar
LibraryMenuShow_NoLabelChar:
	ld a,SFourSpaces
LibraryMenuShow_LabelChar:
	call BufVPutMap
	ld a,':'
	call BufVPutMap
	call BufVPutBlank
	call LibraryMenuGetOffset
	jp ImportMenuShow

LibraryMenuPrev:
	ld a,h
	or l
	dec hl
	ret nz
	scf
	ret

LibraryMenuNext:
	inc hl
libraryMenuCount equ $ + 1
	ld de,0
	or a
	sbc hl,de
	add hl,de
	ccf
	ret

LibraryMenuInvCursor:
	push af
	 call XBufCpy
	 pop af
	ld h,a
	add a,6
	ld d,a
	ld l,22
	ld e,28
	BCALL _InvertRect
	ret

    END_PLUGIN
